# list of all tiles available according to:
# http://landweb.nascom.nasa.gov/cgi-bin/browse/single.cgi?prod=MOD09A1&sat_ver=Terra_C5&date=2006297&imgProd=MOD09A1&proj=HAM
#
# order: horizontal, vertical
available_tiles <- rbind(
 cbind(13:22,1),

 cbind(10:25,2),

 cbind(7:15,3),
 cbind(17:28,3),

 cbind(8:14,4),
 cbind(17:28,4),
 
 cbind(7:13,5),
 cbind(15:31,5),
 
 cbind(2:4,6),
 cbind(7:9,6),
 cbind(16:32,6),
 
 cbind(1:4,7),
 cbind(6:12,7),
 cbind(15:35,7),
 
 cbind(0:2,8),
 cbind(8:13,8),
 cbind(16:23,8),
 cbind(25:35,8),
 
 cbind(0:5,9),
 cbind(8:14,9),
 cbind(16,9),
 cbind(18:23,9),
 cbind(25,9),
 cbind(27:35,9),
 
 cbind(1:6,10),
 cbind(10:14,10),
 cbind(17,10),
 cbind(19:23,10),
 cbind(26:34,10),
 
 cbind(3:7,11),
 cbind(11:16,11),
 cbind(19:23,11),
 cbind(26:32,11),
 
 cbind(11:13,12),
 cbind(16:17,12),
 cbind(19:20,12),
 cbind(23:30,12),
 
 cbind(7:10,13),
 cbind(12:13,13),
 cbind(19:22,13),
 cbind(25:28,13),
 
 cbind(14:17,14),
 cbind(18,14),
 cbind(20:22,14),
 cbind(24:25,14)
)


checkModis<-function(TLUT,year,doyuse) {
 
  intervals <- doyuse-year*1000

  # generate list of needed files
  neededFiles <- array(dim=c(length(TLUT)/6,length(intervals)))
  for (i in 1:(length(TLUT)/6)) {

   if ((length(TLUT)/6)==1) {
    if (TLUT[1]<10) H <- paste("0",TLUT[1],sep="") else H <- TLUT[1]
    if (TLUT[2]<10) V <- paste("0",TLUT[2],sep="") else V <- TLUT[2]
   } else {
    if (TLUT[i,1]<10) H <- paste("0",TLUT[i,1],sep="") else H <- TLUT[i,1]
    if (TLUT[i,2]<10) V <- paste("0",TLUT[i,2],sep="") else V <- TLUT[i,2]
   }

   for (j in 1:length(intervals)) {
    day <- intervals[j]
    if (day<10)       DAY <- paste("00",day,sep="")
    else if (day<100) DAY <- paste("0",day,sep="")
    else              DAY <- day    
    neededFiles[i,j]      <- paste("MOD09A1.A",year,DAY,".h",H,"v",V,sep="")
   }
  }

  # check if files are in modisrep
  if (length(modisrep)!=0) {
   for (i in 1:dim(neededFiles)[1]) {
    for (j in 1:dim(neededFiles)[2]) {
     if (is.na(neededFiles[i,j])) next
     if (length(list.files(path=paste(modisrep,year,sep="/"),pattern=neededFiles[i,j]))>0)
       neededFiles[i,j]<-NA
    }
   }
  }

  # download missing files to via http to mpath
  notavail<-NULL
  if (length(neededFiles[!is.na(neededFiles)])!=0&as.logical(ftdata)) {
    require(RCurl)
    require(rgdal)
    MOD09A1 <- "http://e4ftl01.cr.usgs.gov/MOLT/MOD09A1.005/"
    # get the list of directories
    items <- strsplit(getURL(MOD09A1), "\n")[[1]]
    # you get the folders (and files) but the folder names are in the form of a unix directory listing
    dates<-substring(items[substring(items,46,50)=="href="&substring(items,52,55)==year],52,61)
    julds<-julian(as.numeric(substring(dates,6,7)),
                  as.numeric(substring(dates,9,10)),
                  as.numeric(substring(dates,1,4)),c(1,0,year)) #available juldays 
    nfs<-neededFiles[!is.na(neededFiles)]
    juldsn<-as.numeric(substring(nfs,14,16)) #needed juldays
    for(juld in unique(juldsn)){
      #check if date is available as directory
      if(!juld%in%julds){print(paste("getMODIS:",nfs[juldsn==juld],"not available"));next} #date not available
      daten<-dates[julds==juld]
      for(nf in nfs[juldsn==juld]){
        getlist <- strsplit(getURL(paste(MOD09A1, dates[julds==as.numeric(substring(nf,14,16))], "/", sep=""), .opts=curlOptions(ftplistonly=TRUE)), "\n")[[1]]
        getlist <- substring(getlist[substring(getlist,47,51)=="href="],53,97)
        fnm <- getlist[grep(getlist, pattern=paste(nf,".*.hdf",sep=""))[1]]
        if(is.na(fnm)){print(paste("getMODIS:",nf,"not available"));notavail<-c(notavail,nf);next} #date not available
        print(paste("downloading",fnm))
        download.file(paste(MOD09A1, daten, "/", fnm,sep=""), destfile=paste(modisrep,year,fnm, sep="/"), mode='wb', method='wget', quiet=T, cacheOK=FALSE)
      }
    }
  }

  # check again if files are in modisrep
  if (length(modisrep)!=0) {
   for (i in 1:dim(neededFiles)[1]) {
    for (j in 1:dim(neededFiles)[2]) {
     if (is.na(neededFiles[i,j])) next
     if (length(list.files(path=paste(modisrep,year,sep="/"),pattern=neededFiles[i,j]))>0)
       neededFiles[i,j]<-NA
    }
   }
  }
  # check if everything is there otherwise stop processing
  if (any(!(neededFiles[!is.na(neededFiles)]%in%notavail))) {
   print("Missing files: ")
   print(neededFiles[!is.na(neededFiles)])
   stop("Unable to retrieve all MODIS data needed. Aborting.");
  }  
  
  return(NULL)
}




getModis<-function(doyuse) {

  # load projection info
  projinfo <- getr("projinfo",datp)

  # get coordinates from WRF
  filename <- paste(wrfinput,"/wrfinput_d01",sep="")

  library(RNetCDF)
  nc       <- open.nc(filename)
  lat.wrf  <- t(var.get.nc(nc,"XLAT"))
  lon.wrf  <- t(var.get.nc(nc,"XLONG"))  
  close.nc(nc)
   
  ll    <- c(1,2) 
  ur    <- c(1,2)
  ll[1] <- min(lon.wrf)-1
  ll[2] <- min(lat.wrf)-1  
  ur[1] <- max(lon.wrf)+1
  ur[2] <- max(lat.wrf)+1  

  lons <- seq(from=ll[1],to=ur[1])
  lats <- seq(from=ll[2],to=ur[2])
  tiles <- array()
  for (x in 1:length(lons)) {
   for (y in 1:length(lats)) {
    tile <- system(paste(ldope,"bin/tile_id -proj=SIN -lon=",lons[x]," -lat=",lats[y],sep=""),intern=TRUE)
    tile <- as.numeric(strsplit(tile,split=" ")[[1]][5:6]) # tile[1] = h; tile[2] = v
    if (x==1) tiles <- tile
    else tiles <- rbind(tiles,tile)
   }
  }
  tiles <- unique(tiles)
  tiles <- tiles[order(tiles[,1]),]

  # only select available tiles
  if ((length(tiles)/2)==1) {
   available <- FALSE
   for (j in 1:dim(available_tiles)[1]) {
    if (sum(tiles==available_tiles[j,])==2) available <- TRUE
   }
   if (!available) return(NULL)
  } else {
   newtiles <- array()
   for (i in 1:dim(tiles)[1]) {
    for (j in 1:dim(available_tiles)[1]) {
     if (sum(tiles[i,]==available_tiles[j,])==2) newtiles <- rbind(newtiles,tiles[i,])
    }
   }
   tiles <- newtiles[2:dim(newtiles)[1],]
  }
  
  if (file.exists(paste(scriptpath,"/tiles.txt",sep=""))) {
   file.remove(paste(scriptpath,"/tiles.txt",sep=""))
  }
  
  # create tile lookup table TLUT
  if ((length(tiles)/2)==1) {
   TLUT <- c(tiles[1],tiles[2],0,0,0,0)
   if (tiles[1]<10) H <- paste("0",tiles[1],sep="") else H <- as.character(tiles[1])
   if (tiles[2]<10) V <- paste("0",tiles[2],sep="") else V <- as.character(tiles[2])
   system(paste("echo '",H,",",V,"' >> ",scriptpath,"/tiles.txt",sep=""))
  } else {
   for (line in 1:dim(tiles)[1]) {
     i <- tiles[line,1]
     j <- tiles[line,2]
     print(paste("h",i," v",j,sep=""))
     if (i<10) H <- paste("0",i,sep="") else H <- as.character(i)
     if (j<10) V <- paste("0",j,sep="") else V <- as.character(j)
     system(paste("echo '",H,",",V,"' >> ",scriptpath,"/tiles.txt",sep=""))
     # create tile table    
     if (line!=1) TLUT <- rbind(TLUT,c(i,j,0,0,0,0))
     else TLUT <- as.array(c(i,j,0,0,0,0))
   }

   # sort TLUT
   TLUT <- TLUT[order(TLUT[,2]),]
   TLUT <- TLUT[order(TLUT[,1]),]
  }
  
  # use this code in case you specified tiles to use in text file
#  tiles <- read.table(paste(scriptpath,"/tiles.txt",sep=""))
#  TLUT <- array(dim=c(6))
#  for (i in 1:length(tiles[[1]])) {
#   v    <- as.numeric(strsplit(strsplit(as.character(tiles[[1]][i]),"\\.")[[1]][3],"v")[[1]][2])
#   h    <- as.numeric(strsplit(strsplit(strsplit(as.character(tiles[[1]][i]),"\\.")[[1]][3],"v")[[1]][1],"h")[[1]][2])
#   TLUT <- rbind(TLUT,as.array(c(h,v,0,0,0,0))) 
#  }
#  TLUT <- TLUT[2:dim(TLUT)[1],]
#  TLUT <- TLUT[order(TLUT[,2]),]
#  TLUT <- TLUT[order(TLUT[,1]),]

  # make sure all MODIS data is available
  checkModis(TLUT,projinfo$year,doyuse)
  
  return(TLUT)
}

