getMinMat <- function(matA,matB) {

 # get minimum matrix
 matA[is.na(matA)]    <- 9999        # neutral to min
 matB[is.na(matB)]    <- 9999        
 sel                  <- matB < matA
 matA                 <- matA * !sel # set A 0 where B < A
 matB                 <- matB *  sel # set B 0 where B > A
 minMat               <- matA + matB # join minima
 minMat[minMat==9999] <- NA          # couldn't be replaced

 return(minMat)
}

getMaxMat <- function(matA,matB) {

 # get maximum matrix
 matA[is.na(matA)]    <- -9999       # neutral to max
 matB[is.na(matB)]    <- -9999       
 sel                  <- matB > matA
 matA                 <- matA * !sel # set A 0 where B > A
 matB                 <- matB *  sel # set B 0 where B < A
 maxMat               <- matA + matB # join maxima
 maxMat[maxMat==-9999]<- NA          # couldn't be replaced

 return(maxMat)
}


getMinMaxIndices <- function(vari,year,dom,doyuse) {
  # get minimum indices map
  # to do: take flexible snow times into account

  # check if output is already existing
  filename_min <- paste(datp,".RData",vari,".minAggregatedClassed.",year,".",dom,sep="")
  filename_max <- paste(datp,".RData",vari,".maxAggregatedClassed.",year,".",dom,sep="")
  if (file.exists(filename_min) & file.exists(filename_max)) return(NULL)

  indices <- getr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,sep=""),datp)
  max <- min <- array(dim=c(dim(indices)[1],dim(indices)[2],dim(indices)[4]))
  
  # get time range where there are no index > 1 at all
  # do this only for evi indices
  start <- -1
  end   <- -1
  if (vari == "evi") { 
   for (i in 1:dim(indices)[3]) {
    toolarge_num <- length(indices[,,i,][indices[,,i,]>1&!is.na(indices[,,i,])])
    print(paste(i,toolarge_num))
    if (toolarge_num==0) {
     if (start==-1) start <- i
    } else {
     if (start!=-1 & end==-1) end <- i-1
    }
    
   }
  } else {
   start <- 1
   end   <- dim(indices)[3]
  }
  
  if (end==-1) end <- dim(indices)[3]

  for (v in 1:dim(indices)[4]) {       # vegetation classes
    for (t in 1:dim(indices)[3]) {     # time
      min[,,v] <- getMinMat(min[,,v],indices[,,t,v])
      if (t>=start & t<=end & vari=="evi") max[,,v] <- getMaxMat(max[,,v],indices[,,t,v])
      if (vari=="lswi") max[,,v] <- getMaxMat(max[,,v],indices[,,t,v])
    }
  }

  assignr(paste(vari,".minAggregatedClassed.",year,".",dom,sep=""),min,datp)
  assignr(paste(vari,".maxAggregatedClassed.",year,".",dom,sep=""),max,datp)
  
  
  return(NULL)
}



