#!/bin/bash
# script to start parallel execution of loess filtering on cluster
#
# usage:
# ./submit_getmapping.sh <PATH_TO_PREPROC> <NEST> <LENGTH_OF_LAT_VECTOR>

process_id=$$

job_limit=10 #number of jobs that should run in parallel

dom=$2

# calculate index for each job
lat_length=$3
count=$(($lat_length/$job_limit))
remainder=$(($lat_length%$job_limit))

cd $1

for ((i=1;i<=$(($lat_length/$count));i++))
do

 start_index=$(($(($i-1))*$count+1))

 command="R --no-save --no-readline --args ${dom} ${start_index} ${count} < lsf_getmapping.r >& lsf_getmapping.r.Rout.${i}"
 bsub -J getmapping_${process_id} -o lsf_getmapping.r.Rout.${i} -e lsf_getmapping.r.Rout.${i} -R "rusage[mem=4000]" -q all -g /rkretsch_group ${command}

done

i=$(($lat_length/$count+1))

# submit the remainder
if [[ "$remainder" -ne "0" ]]
then

 start_index=$(($lat_length-$remainder+1))

 command="R --no-save --no-readline --args ${dom} ${start_index} ${remainder} < lsf_getmapping.r >& lsf_getmapping.r.Rout.${i}"
 bsub -J getmapping_${process_id} -o lsf_getmapping.r.Rout.${i} -e lsf_getmapping.r.Rout.${i} -R "rusage[mem=4000]" -q all -g /rkretsch_group ${command}
 
fi



# place another job to make sure all the loess jobs have finished when this script ends
bsub -w "done(getmapping_${process_id})" -q all -K -g /rkretsch_group sleep 1
